/**
 * @operatorCom floatMultiply, floatDivide, floatAdd, floatSub
 * @description 解决js运算精度丢失的公共方法
 * @example
 * ```
 * var operatorCom = require('../common/operator.js');
 * operatorCom.floatMultiply(0.0023,100);   // 结果为0.23 ；而直接用*则会出现精度丢失问题：0.0023*100=0.22999999999999998
 * ```
 */

/**
 * 乘法 - js运算精度丢失问题
 * @param arg1  数1
 * @param arg2  数2
 */
var floatMultiply = (arg1, arg2)=> {
	if(arg1 == null || arg2 == null){
		return null;
	}
	var n1,n2;
	var r1, r2; // 小数位数
	try {
		r1 = arg1.toString().split(".")[1].length;
	} catch (e) {
		r1 = 0;
	}
	try {
		r2 = arg2.toString().split(".")[1].length;
	} catch (e) {
		r2 = 0;
	}
	n1 = Number(arg1.toString().replace(".", ""));
	n2 = Number(arg2.toString().replace(".", ""));
	return n1 * n2 / Math.pow(10, r1+r2);
}

/**
 * 除法 - js运算精度丢失问题
 * @param arg1  数1
 * @param arg2  数2
 */
var floatDivide = (arg1, arg2)=> {
	if(arg1 == null){
		return null;
	}
	if(arg2 == null || arg2 == 0){
		return null;
	}
	var n1,n2;
	var r1, r2; // 小数位数
	try {
		r1 = arg1.toString().split(".")[1].length;
	} catch (e) {
		r1 = 0;
	}
	try {
		r2 = arg2.toString().split(".")[1].length;
	} catch (e) {
		r2 = 0;
	}
	n1 = Number(arg1.toString().replace(".", ""));
	n2 = Number(arg2.toString().replace(".", ""));
	return floatMultiply((n1 / n2), Math.pow(10, r2-r1));
	// return (n1 / n2) * Math.pow(10, r2 - r1);   // 直接乘法还是会出现精度问题
}

/**
 * 加法 - js运算精度丢失问题
 * @param arg1  数1
 * @param arg2  数2
 */
var floatAdd = (arg1, arg2)=> {
	var r1, r2, m;
	try {
		r1 = arg1.toString().split(".")[1].length;
	} catch (e) {
		r1 = 0;
	}
	try {
		r2 = arg2.toString().split(".")[1].length;
	} catch (e) {
		r2 = 0;
	}
	m = Math.pow(10, Math.max(r1, r2));
	return (floatMultiply(arg1 , m) + floatMultiply(arg2 , m)) / m;
}
 
/**
 * 减法 - js运算精度丢失问题
 * @param arg1  数1
 * @param arg2  数2
 */
var floatSub = (arg1, arg2)=> {
	var r1, r2, m, n;
	try {
		r1 = arg1.toString().split(".")[1].length;
	} catch (e) {
		r1 = 0;
	}
	try {
		r2 = arg2.toString().split(".")[1].length;
	} catch (e) {
		r2 = 0;
	}
	m = Math.pow(10, Math.max(r1, r2));
	// 动态控制精度长度
	n = (r1 >= r2) ? r1 : r2;
	return ((floatMultiply(arg1 , m) - floatMultiply(arg2 , m)) / m).toFixed(n);
}
var operatorCom = {
    floatMultiply: floatMultiply,
    floatDivide: floatDivide,
    floatAdd: floatAdd,
    floatSub: floatSub
}
module.exports = operatorCom;